@@ -10,17 +10,16 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-__all__ = [
-    'list_opts',
-    'set_defaults',
-]
-
 import copy
 
 from oslo_config import cfg
 
 from oslo_policy._i18n import _
 
+__all__ = [
+    'list_opts',
+    'set_defaults',
+]
 
 _option_group = 'oslo_policy'
 
@@ -120,4 +119,4 @@ def set_defaults(conf, policy_file=None):
     _register(conf)
 
     if policy_file is not None:
-        conf.set_default('policy_file', policy_file, group=_option_group)
+        cfg.set_defaults(_options, policy_file=policy_file)
