@@ -10,18 +10,18 @@
 #    under the License.
 
 import operator
-import sys
+import warnings
 
-import fixtures
 import mock
 from oslo_config import cfg
-from six import moves
 import stevedore
 import testtools
+import yaml
 
 from oslo_policy import generator
 from oslo_policy import policy
 from oslo_policy.tests import base
+from oslo_serialization import jsonutils
 
 
 OPTS = {'base_rules': [policy.RuleDefault('admin', 'is_admin:True',
@@ -49,10 +49,6 @@ class GenerateSampleYAMLTestCase(base.PolicyBaseTestCase):
         super(GenerateSampleYAMLTestCase, self).setUp()
         self.enforcer = policy.Enforcer(self.conf, policy_file='policy.yaml')
 
-    def _capture_stdout(self):
-        self.useFixture(fixtures.MonkeyPatch('sys.stdout', moves.StringIO()))
-        return sys.stdout
-
     def test_generate_loadable_yaml(self):
         extensions = []
         for name, opts in OPTS.items():
@@ -227,10 +223,9 @@ class GenerateSampleYAMLTestCase(base.PolicyBaseTestCase):
         expected = '''# Create a bar.
 #"foo:create_bar": "role:fizz"
 
-# DEPRECATED
-# "foo:post_bar":"role:fizz" has been deprecated since N in favor of
-# "foo:create_bar":"role:fizz".
-# foo:post_bar is being removed in favor of foo:create_bar
+# DEPRECATED "foo:post_bar":"role:fizz" has been deprecated since N in
+# favor of "foo:create_bar":"role:fizz". foo:post_bar is being removed
+# in favor of foo:create_bar
 "foo:post_bar": "rule:foo:create_bar"
 '''
         stdout = self._capture_stdout()
@@ -244,26 +239,15 @@ class GenerateSampleYAMLTestCase(base.PolicyBaseTestCase):
             )
         self.assertEqual(expected, stdout.getvalue())
 
-    def test_empty_line_formatting(self):
+    def _test_formatting(self, description, expected):
         rule = [policy.RuleDefault('admin', 'is_admin:True',
-                                   description='Check Summary \n'
-                                   '\n'
-                                   'This is a description to '
-                                   'check that empty line has '
-                                   'no white spaces.')]
+                                   description=description)]
         ext = stevedore.extension.Extension(name='check_rule',
                                             entry_point=None,
                                             plugin=None, obj=rule)
         test_mgr = stevedore.named.NamedExtensionManager.make_test_instance(
             extensions=[ext], namespace=['check_rule'])
 
-        # no whitespace on empty line
-        expected = '''# Check Summary
-#
-# This is a description to check that empty line has no white spaces.
-#"admin": "is_admin:True"
-
-'''
         output_file = self.get_config_file_fullname('policy.yaml')
         with mock.patch('stevedore.named.NamedExtensionManager',
                         return_value=test_mgr) as mock_ext_mgr:
@@ -278,16 +262,82 @@ class GenerateSampleYAMLTestCase(base.PolicyBaseTestCase):
 
         self.assertEqual(expected, written_policy)
 
+    def test_empty_line_formatting(self):
+        description = ('Check Summary \n'
+                       '\n'
+                       'This is a description to '
+                       'check that empty line has '
+                       'no white spaces.')
+        expected = """# Check Summary
+#
+# This is a description to check that empty line has no white spaces.
+#"admin": "is_admin:True"
+
+"""
+
+        self._test_formatting(description, expected)
+
+    def test_paragraph_formatting(self):
+        description = """
+Here's a neat description with a paragraph. We want to make sure that it wraps
+properly.
+"""
+        expected = """# Here's a neat description with a paragraph. We want \
+to make sure
+# that it wraps properly.
+#"admin": "is_admin:True"
+
+"""
+
+        self._test_formatting(description, expected)
+
+    def test_literal_block_formatting(self):
+        description = """Here's another description.
+
+    This one has a literal block.
+    These lines should be kept apart.
+    They should not be wrapped, even though they may be longer than 70 chars
+"""
+        expected = """# Here's another description.
+#
+#     This one has a literal block.
+#     These lines should be kept apart.
+#     They should not be wrapped, even though they may be longer than 70 chars
+#"admin": "is_admin:True"
+
+"""
+
+        self._test_formatting(description, expected)
+
+    def test_invalid_formatting(self):
+        description = """Here's a broken description.
+
+We have some text...
+    Followed by a literal block without any spaces.
+    We don't support definition lists, so this is just wrong!
+"""
+        expected = """# Here's a broken description.
+#
+# We have some text...
+#
+#     Followed by a literal block without any spaces.
+#     We don't support definition lists, so this is just wrong!
+#"admin": "is_admin:True"
+
+"""
+
+        with warnings.catch_warnings(record=True) as warns:
+            self._test_formatting(description, expected)
+            self.assertEqual(1, len(warns))
+            self.assertTrue(issubclass(warns[-1].category, FutureWarning))
+            self.assertIn('Invalid policy description', str(warns[-1].message))
+
 
 class GenerateSampleJSONTestCase(base.PolicyBaseTestCase):
     def setUp(self):
         super(GenerateSampleJSONTestCase, self).setUp()
         self.enforcer = policy.Enforcer(self.conf, policy_file='policy.json')
 
-    def _capture_stdout(self):
-        self.useFixture(fixtures.MonkeyPatch('sys.stdout', moves.StringIO()))
-        return sys.stdout
-
     def test_generate_loadable_json(self):
         extensions = []
         for name, opts in OPTS.items():
@@ -471,10 +521,6 @@ class ListRedundantTestCase(base.PolicyBaseTestCase):
     def setUp(self):
         super(ListRedundantTestCase, self).setUp()
 
-    def _capture_stdout(self):
-        self.useFixture(fixtures.MonkeyPatch('sys.stdout', moves.StringIO()))
-        return sys.stdout
-
     def test_matched_rules(self):
         extensions = []
         for name, opts in OPTS.items():
@@ -528,3 +574,104 @@ class ListRedundantTestCase(base.PolicyBaseTestCase):
         opt1 = matches[1]
         self.assertEqual('"owner"', opt1[0])
         self.assertEqual('"project_id:%(project_id)s"', opt1[1])
+
+
+class UpgradePolicyTestCase(base.PolicyBaseTestCase):
+    def setUp(self):
+        super(UpgradePolicyTestCase, self).setUp()
+        policy_json_contents = jsonutils.dumps({
+            "deprecated_name": "rule:admin"
+        })
+        self.create_config_file('policy.json', policy_json_contents)
+        deprecated_policy = policy.DeprecatedRule(
+            name='deprecated_name',
+            check_str='rule:admin'
+        )
+        self.new_policy = policy.DocumentedRuleDefault(
+            name='new_policy_name',
+            check_str='rule:admin',
+            description='test_policy',
+            operations=[{'path': '/test', 'method': 'GET'}],
+            deprecated_rule=deprecated_policy,
+            deprecated_reason='test',
+            deprecated_since='Stein'
+        )
+        self.extensions = []
+        ext = stevedore.extension.Extension(name='test_upgrade',
+                                            entry_point=None,
+                                            plugin=None,
+                                            obj=[self.new_policy])
+        self.extensions.append(ext)
+
+    def test_upgrade_policy_json_file(self):
+        test_mgr = stevedore.named.NamedExtensionManager.make_test_instance(
+            extensions=self.extensions, namespace='test_upgrade')
+        with mock.patch('stevedore.named.NamedExtensionManager',
+                        return_value=test_mgr):
+            testargs = ['olsopolicy-policy-upgrade',
+                        '--policy',
+                        self.get_config_file_fullname('policy.json'),
+                        '--namespace', 'test_upgrade',
+                        '--output-file',
+                        self.get_config_file_fullname('new_policy.json'),
+                        '--format', 'json']
+            with mock.patch('sys.argv', testargs):
+                generator.upgrade_policy()
+                new_file = self.get_config_file_fullname('new_policy.json')
+                new_policy = jsonutils.loads(open(new_file, 'r').read())
+                self.assertIsNotNone(new_policy.get('new_policy_name'))
+                self.assertIsNone(new_policy.get('deprecated_name'))
+
+    def test_upgrade_policy_yaml_file(self):
+        test_mgr = stevedore.named.NamedExtensionManager.make_test_instance(
+            extensions=self.extensions, namespace='test_upgrade')
+        with mock.patch('stevedore.named.NamedExtensionManager',
+                        return_value=test_mgr):
+            testargs = ['olsopolicy-policy-upgrade',
+                        '--policy',
+                        self.get_config_file_fullname('policy.json'),
+                        '--namespace', 'test_upgrade',
+                        '--output-file',
+                        self.get_config_file_fullname('new_policy.yaml'),
+                        '--format', 'yaml']
+            with mock.patch('sys.argv', testargs):
+                generator.upgrade_policy()
+                new_file = self.get_config_file_fullname('new_policy.yaml')
+                new_policy = yaml.safe_load(open(new_file, 'r'))
+                self.assertIsNotNone(new_policy.get('new_policy_name'))
+                self.assertIsNone(new_policy.get('deprecated_name'))
+
+    def test_upgrade_policy_json_stdout(self):
+        test_mgr = stevedore.named.NamedExtensionManager.make_test_instance(
+            extensions=self.extensions, namespace='test_upgrade')
+        stdout = self._capture_stdout()
+        with mock.patch('stevedore.named.NamedExtensionManager',
+                        return_value=test_mgr):
+            testargs = ['olsopolicy-policy-upgrade',
+                        '--policy',
+                        self.get_config_file_fullname('policy.json'),
+                        '--namespace', 'test_upgrade',
+                        '--format', 'json']
+            with mock.patch('sys.argv', testargs):
+                generator.upgrade_policy()
+                expected = '''{
+    "new_policy_name": "rule:admin"
+}'''
+                self.assertEqual(expected, stdout.getvalue())
+
+    def test_upgrade_policy_yaml_stdout(self):
+        test_mgr = stevedore.named.NamedExtensionManager.make_test_instance(
+            extensions=self.extensions, namespace='test_upgrade')
+        stdout = self._capture_stdout()
+        with mock.patch('stevedore.named.NamedExtensionManager',
+                        return_value=test_mgr):
+            testargs = ['olsopolicy-policy-upgrade',
+                        '--policy',
+                        self.get_config_file_fullname('policy.json'),
+                        '--namespace', 'test_upgrade',
+                        '--format', 'yaml']
+            with mock.patch('sys.argv', testargs):
+                generator.upgrade_policy()
+                expected = '''new_policy_name: rule:admin
+'''
+                self.assertEqual(expected, stdout.getvalue())
